import os

from rp.libs.parso import file_io


class AbstractFolderIO(object):
    def __init__(self, path):
        self.path = path

    def list(self):
        raise NotImplementedError

    def get_file_io(self, name):
        raise NotImplementedError


class FolderIO(AbstractFolderIO):
    def list(self):
        return os.listdir(self.path)

    def get_file_io(self, name):
        return FileIO(os.path.join(self.path, name))


class FileIOFolderMixin(object):
    def get_parent_folder(self):
        return FolderIO(os.path.dirname(self.path))


class ZipFileIO(file_io.KnownContentFileIO, FileIOFolderMixin):
    """For .zip and .egg archives"""
    def __init__(self, path, code, zip_path):
        super(ZipFileIO, self).__init__(path, code)
        self._zip_path = zip_path

    def get_last_modified(self):
        try:
            return os.path.getmtime(self._zip_path)
        except OSError:  # Python 3 would probably only need FileNotFoundError
            return None


class FileIO(file_io.FileIO, FileIOFolderMixin):
    pass


class KnownContentFileIO(file_io.KnownContentFileIO, FileIOFolderMixin):
    pass



import urwid
from loguru import logger

from .view_frame import ViewFrame


class View:
    """
    pyfx UI entry point.

    It manages the UI thread and switches views between components with
    keypress.

    .. data:`palette`
       The current theme defined in `pyfx`.
    """

    def __init__(self, config, client):
        """
        """
        self._config = config
        self._frame = ViewFrame(client, self, self._config.keymap.mapping)

        self._input_filter = self._config.keymap.mapping.input_filter

        self._screen = None
        self._loop = None

    def run(self, data):
        """
        To start the :py:class:`urwid.MainLoop`.

        :param data: the current JSON data
        :return:
        """
        self._frame.set_data(data)
        self._screen = urwid.raw_display.Screen(input=open('/dev/tty'))
        self._screen.tty_signal_keys('undefined', 'undefined', 'undefined',
                                     'undefined', 'undefined')
        self._loop = urwid.MainLoop(
            self._frame,
            self._config.appearance.color_scheme,
            pop_ups=True,
            screen=self._screen,
            input_filter=self._input_filter.filter,
            unhandled_input=self.unhandled_input
        )

        # noinspection PyBroadException
        try:
            self._loop.run()
        except Exception as e:
            logger.opt(exception=True).\
                error("Unknown exception encountered, exit with {}", e)
            self._screen.clear()

    def process_input(self, data, keys):
        """
        Test-used only to process a list of keypress
        """
        self._frame.set_data(data)
        self._screen = urwid.raw_display.Screen()
        self._loop = urwid.MainLoop(
            self._frame,
            self._config.appearance.color_scheme,
            pop_ups=True,
            screen=self._screen,
            input_filter=self._input_filter.filter,
            unhandled_input=self.unhandled_input
        )

        try:
            for index, key in enumerate(keys):
                # work around for urwid.MainLoop#process_input does not apply
                # input filter
                key = self._loop.input_filter([key], None)
                if len(key) >= 1 and (not self._loop.process_input(key)):
                    return False, f"keys[{index}]: {key} is not handled"
        except urwid.ExitMainLoop:
            pass
        finally:
            self._screen.clear()
        return True, ""

    def size(self):
        return self._screen.get_cols_rows()

    def exit(self, exception=None):
        if not self._loop:
            return
        if exception:
            raise urwid.ExitMainLoop(exception)
        raise urwid.ExitMainLoop()

    def unhandled_input(self, k):
        if k == self._config.keymap.mapping.exit:
            self.exit()



from typing import Any

fastbinary: Any

class ResultCode:
    OK: Any
    TRY_LATER: Any

class LogEntry:
    thrift_spec: Any
    category: Any
    message: Any
    def __init__(self, category=..., message=...) -> None: ...
    def read(self, iprot): ...
    def write(self, oprot): ...
    def validate(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
